@@ -1,8 +1,38 @@
 #!/usr/bin/env python
+"""
+This module is designed to fetch your external IP address from the internet.
+It is used mostly when behind a NAT.
+It picks your IP randomly from a serverlist to minimize request
+overhead on a single server
+
+If you want to add or remove your server from the list contact me on github
+
+
+API Usage
+=========
+
+    >>> import ipgetter
+    >>> myip = ipgetter.myip()
+    >>> myip
+    '8.8.8.8'
+
+    >>> ipgetter.IPgetter().test()
+
+    Number of servers: 47
+    IP's :
+    8.8.8.8 = 47 ocurrencies
+
+
+Copyright © 2014 phoemur@gmail.com
+This work is free. You can redistribute it and/or modify it under the
+terms of the Do What The Fuck You Want To Public License, Version 2,
+as published by Sam Hocevar. See http://www.wtfpl.net/ for more details.
+"""
 
 import re
 import random
 import signal
+
 from sys import version_info
 from functools import wraps
 
@@ -13,16 +43,17 @@ if PY3K:
 else:
     import urllib2 as urllib
 
-__version__ = "0.4"
+__version__ = "0.5.1"
 
 
-def timeout(seconds, error_message = 'Function call timed out'):
+def timeout(seconds, error_message='Function call timed out'):
     '''
     Decorator that provides timeout to a function
     '''
     def decorated(func):
         def _handle_timeout(signum, frame):
             raise TimeoutError(error_message)
+
         @wraps(func)
         def wrapper(*args, **kwargs):
             signal.signal(signal.SIGALRM, _handle_timeout)
@@ -35,18 +66,21 @@ def timeout(seconds, error_message = 'Function call timed out'):
         return wrapper
     return decorated
 
+
 @timeout(120)
 def myip():
     return IPgetter().get_externalip()
 
+
 class IPgetter(object):
+
     '''
     This class is designed to fetch your external IP address from the internet.
     It is used mostly when behind a NAT.
     It picks your IP randomly from a serverlist to minimize request overhead
     on a single server
     '''
-    
+
     def __init__(self):
         self.server_list = ['http://ip.dnsexit.com',
                             'http://ifconfig.me/ip',
@@ -57,12 +91,10 @@ class IPgetter(object):
                             'http://ip.my-proxy.com/',
                             'http://websiteipaddress.com/WhatIsMyIp',
                             'http://getmyipaddress.org/',
-                            'http://showmyipaddress.com/',
                             'http://www.my-ip-address.net/',
                             'http://myexternalip.com/raw',
                             'http://www.canyouseeme.org/',
                             'http://www.trackip.net/',
-                            'http://myip.dnsdynamic.org/',
                             'http://icanhazip.com/',
                             'http://www.iplocation.net/',
                             'http://www.howtofindmyipaddress.com/',
@@ -82,6 +114,7 @@ class IPgetter(object):
                             'http://www.myipnumber.com/my-ip-address.asp',
                             'http://www.whatsmyipaddress.net/',
                             'http://formyip.com/',
+                            'https://check.torproject.org/',
                             'http://www.displaymyip.com/',
                             'http://www.bobborst.com/tools/whatsmyip/',
                             'http://www.geoiptool.com/',
@@ -91,13 +124,15 @@ class IPgetter(object):
                             'http://myexternalip.com/',
                             'http://www.ip-adress.eu/',
                             'http://www.infosniper.net/',
-                            'http://wtfismyip.com/']
-        
+                            'http://wtfismyip.com/',
+                            'http://ipinfo.io/',
+                            'http://httpbin.org/ip']
+
     def get_externalip(self):
         '''
         This function gets your IP from a random server
         '''
-        
+
         random.shuffle(self.server_list)
         myip = ''
         for server in self.server_list:
@@ -106,20 +141,31 @@ class IPgetter(object):
                 return myip
             else:
                 continue
-        return ''        
-       
+        return ''
+
     def fetch(self, server):
         '''
         This function gets your IP from a specific server
         '''
-        
+        url = None
         opener = urllib.build_opener()
-        opener.addheaders = [('User-agent', "Mozilla/5.0")]
-        
+        opener.addheaders = [('User-agent',
+                              "Mozilla/5.0 (X11; Linux x86_64; rv:24.0) Gecko/20100101 Firefox/24.0")]
+
         try:
             url = opener.open(server)
-            content = url.read().decode('utf-8') if PY3K else url.read()
-            m = re.search('(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)',content)
+            content = url.read()
+
+            # Didn't want to import chardet. Prefered to stick to stdlib
+            if PY3K:
+                try:
+                    content = content.decode('UTF-8')
+                except UnicodeDecodeError:
+                    content = content.decode('ISO-8859-1')
+
+            m = re.search(
+                '(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)',
+                content)
             myip = m.group(0)
             return myip if len(myip) > 0 else ''
         except Exception:
@@ -127,22 +173,26 @@ class IPgetter(object):
         finally:
             if url:
                 url.close()
-        
+
     def test(self):
         '''
-        This functions tests the consistency of the servers on the list when retrieving your IP.
+        This functions tests the consistency of the servers
+        on the list when retrieving your IP.
         All results should be the same.
         '''
-        
+
         resultdict = {}
         for server in self.server_list:
-            resultdict.update(**{server:self.fetch(server)})
-            
+            resultdict.update(**{server: self.fetch(server)})
+
         ips = sorted(resultdict.values())
         ips_set = set(ips)
         print('\nNumber of servers: {}'.format(len(self.server_list)))
         print("IP's :")
         for ip, ocorrencia in zip(ips_set, map(lambda x: ips.count(x), ips_set)):
             print('{0} = {1} ocurrenc{2}'.format(ip if len(ip) > 0 else 'broken server', ocorrencia, 'y' if ocorrencia == 1 else 'ies'))
-        print('\n') 
+        print('\n')
         print(resultdict)
+
+if __name__ == '__main__':
+    print(myip())
